begin_version
3
end_version
begin_metric
0
end_metric
77
begin_variable
var0
-1
2
Atom uninitialized()
NegatedAtom uninitialized()
end_variable
begin_variable
var1
-1
2
Atom available(up-rsrc)
NegatedAtom available(up-rsrc)
end_variable
begin_variable
var2
-1
2
Atom available(htmovercolor-rsrc)
NegatedAtom available(htmovercolor-rsrc)
end_variable
begin_variable
var3
-1
2
Atom available(htmoverblack-rsrc)
NegatedAtom available(htmoverblack-rsrc)
end_variable
begin_variable
var4
-1
2
Atom available(finisher2-rsrc)
NegatedAtom available(finisher2-rsrc)
end_variable
begin_variable
var5
-1
2
Atom available(finisher1-rsrc)
NegatedAtom available(finisher1-rsrc)
end_variable
begin_variable
var6
-1
2
Atom available(endcap-rsrc)
NegatedAtom available(endcap-rsrc)
end_variable
begin_variable
var7
-1
2
Atom available(down-rsrc)
NegatedAtom available(down-rsrc)
end_variable
begin_variable
var8
-1
2
Atom available(colorprinter-rsrc)
NegatedAtom available(colorprinter-rsrc)
end_variable
begin_variable
var9
-1
2
Atom available(colorfeeder-rsrc)
NegatedAtom available(colorfeeder-rsrc)
end_variable
begin_variable
var10
-1
2
Atom available(colorcontainer-rsrc)
NegatedAtom available(colorcontainer-rsrc)
end_variable
begin_variable
var11
-1
2
Atom available(blackprinter-rsrc)
NegatedAtom available(blackprinter-rsrc)
end_variable
begin_variable
var12
-1
2
Atom available(blackfeeder-rsrc)
NegatedAtom available(blackfeeder-rsrc)
end_variable
begin_variable
var13
-1
2
Atom available(blackcontainer-rsrc)
NegatedAtom available(blackcontainer-rsrc)
end_variable
begin_variable
var14
-1
2
Atom sideup(sheet1, back)
NegatedAtom sideup(sheet1, back)
end_variable
begin_variable
var15
-1
2
Atom notprintedwith(sheet1, front, black)
NegatedAtom notprintedwith(sheet1, front, black)
end_variable
begin_variable
var16
-1
18
Atom location(sheet1, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet1, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet1, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet1, colorcontainer_entry-down_bottomexit)
Atom location(sheet1, colorcontainer_exit-up_bottomentry)
Atom location(sheet1, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet1, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet1, down_bottomentry-colorfeeder_exit)
Atom location(sheet1, down_topexit-htmovercolor_entry)
Atom location(sheet1, endcap_entry-blackcontainer_exit)
Atom location(sheet1, finisher1_entry-up_topexit)
Atom location(sheet1, finisher2_entry-finisher1_exit)
Atom location(sheet1, finisher2_exit)
Atom location(sheet1, htmoverblack_entry-endcap_exit)
Atom location(sheet1, htmoverblack_exit-down_topentry)
Atom location(sheet1, htmovercolor_exit-up_topentry)
Atom location(sheet1, some_feeder_tray)
Atom location(sheet1, some_finisher_tray)
end_variable
begin_variable
var17
-1
2
Atom notprintedwith(sheet1, back, black)
NegatedAtom notprintedwith(sheet1, back, black)
end_variable
begin_variable
var18
-1
2
Atom notprintedwith(sheet1, back, color)
NegatedAtom notprintedwith(sheet1, back, color)
end_variable
begin_variable
var19
-1
2
Atom notprintedwith(sheet1, front, color)
NegatedAtom notprintedwith(sheet1, front, color)
end_variable
begin_variable
var20
-1
2
Atom sideup(sheet1, front)
NegatedAtom sideup(sheet1, front)
end_variable
begin_variable
var21
-1
2
Atom stackedin(sheet1, finisher1_tray)
NegatedAtom stackedin(sheet1, finisher1_tray)
end_variable
begin_variable
var22
-1
2
Atom sideup(sheet2, back)
NegatedAtom sideup(sheet2, back)
end_variable
begin_variable
var23
-1
2
Atom notprintedwith(sheet2, front, black)
NegatedAtom notprintedwith(sheet2, front, black)
end_variable
begin_variable
var24
-1
18
Atom location(sheet2, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet2, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet2, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet2, colorcontainer_entry-down_bottomexit)
Atom location(sheet2, colorcontainer_exit-up_bottomentry)
Atom location(sheet2, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet2, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet2, down_bottomentry-colorfeeder_exit)
Atom location(sheet2, down_topexit-htmovercolor_entry)
Atom location(sheet2, endcap_entry-blackcontainer_exit)
Atom location(sheet2, finisher1_entry-up_topexit)
Atom location(sheet2, finisher2_entry-finisher1_exit)
Atom location(sheet2, finisher2_exit)
Atom location(sheet2, htmoverblack_entry-endcap_exit)
Atom location(sheet2, htmoverblack_exit-down_topentry)
Atom location(sheet2, htmovercolor_exit-up_topentry)
Atom location(sheet2, some_feeder_tray)
Atom location(sheet2, some_finisher_tray)
end_variable
begin_variable
var25
-1
2
Atom notprintedwith(sheet2, back, black)
NegatedAtom notprintedwith(sheet2, back, black)
end_variable
begin_variable
var26
-1
2
Atom notprintedwith(sheet2, back, color)
NegatedAtom notprintedwith(sheet2, back, color)
end_variable
begin_variable
var27
-1
2
Atom notprintedwith(sheet2, front, color)
NegatedAtom notprintedwith(sheet2, front, color)
end_variable
begin_variable
var28
-1
2
Atom sideup(sheet2, front)
NegatedAtom sideup(sheet2, front)
end_variable
begin_variable
var29
-1
2
Atom stackedin(sheet2, finisher1_tray)
NegatedAtom stackedin(sheet2, finisher1_tray)
end_variable
begin_variable
var30
-1
2
Atom sideup(sheet3, back)
NegatedAtom sideup(sheet3, back)
end_variable
begin_variable
var31
-1
2
Atom notprintedwith(sheet3, front, black)
NegatedAtom notprintedwith(sheet3, front, black)
end_variable
begin_variable
var32
-1
18
Atom location(sheet3, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet3, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet3, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet3, colorcontainer_entry-down_bottomexit)
Atom location(sheet3, colorcontainer_exit-up_bottomentry)
Atom location(sheet3, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet3, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet3, down_bottomentry-colorfeeder_exit)
Atom location(sheet3, down_topexit-htmovercolor_entry)
Atom location(sheet3, endcap_entry-blackcontainer_exit)
Atom location(sheet3, finisher1_entry-up_topexit)
Atom location(sheet3, finisher2_entry-finisher1_exit)
Atom location(sheet3, finisher2_exit)
Atom location(sheet3, htmoverblack_entry-endcap_exit)
Atom location(sheet3, htmoverblack_exit-down_topentry)
Atom location(sheet3, htmovercolor_exit-up_topentry)
Atom location(sheet3, some_feeder_tray)
Atom location(sheet3, some_finisher_tray)
end_variable
begin_variable
var33
-1
2
Atom notprintedwith(sheet3, back, black)
NegatedAtom notprintedwith(sheet3, back, black)
end_variable
begin_variable
var34
-1
2
Atom notprintedwith(sheet3, back, color)
NegatedAtom notprintedwith(sheet3, back, color)
end_variable
begin_variable
var35
-1
2
Atom notprintedwith(sheet3, front, color)
NegatedAtom notprintedwith(sheet3, front, color)
end_variable
begin_variable
var36
-1
2
Atom sideup(sheet3, front)
NegatedAtom sideup(sheet3, front)
end_variable
begin_variable
var37
-1
2
Atom stackedin(sheet3, finisher1_tray)
NegatedAtom stackedin(sheet3, finisher1_tray)
end_variable
begin_variable
var38
-1
2
Atom sideup(sheet4, back)
NegatedAtom sideup(sheet4, back)
end_variable
begin_variable
var39
-1
2
Atom notprintedwith(sheet4, front, black)
NegatedAtom notprintedwith(sheet4, front, black)
end_variable
begin_variable
var40
-1
18
Atom location(sheet4, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet4, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet4, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet4, colorcontainer_entry-down_bottomexit)
Atom location(sheet4, colorcontainer_exit-up_bottomentry)
Atom location(sheet4, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet4, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet4, down_bottomentry-colorfeeder_exit)
Atom location(sheet4, down_topexit-htmovercolor_entry)
Atom location(sheet4, endcap_entry-blackcontainer_exit)
Atom location(sheet4, finisher1_entry-up_topexit)
Atom location(sheet4, finisher2_entry-finisher1_exit)
Atom location(sheet4, finisher2_exit)
Atom location(sheet4, htmoverblack_entry-endcap_exit)
Atom location(sheet4, htmoverblack_exit-down_topentry)
Atom location(sheet4, htmovercolor_exit-up_topentry)
Atom location(sheet4, some_feeder_tray)
Atom location(sheet4, some_finisher_tray)
end_variable
begin_variable
var41
-1
2
Atom notprintedwith(sheet4, back, black)
NegatedAtom notprintedwith(sheet4, back, black)
end_variable
begin_variable
var42
-1
2
Atom notprintedwith(sheet4, back, color)
NegatedAtom notprintedwith(sheet4, back, color)
end_variable
begin_variable
var43
-1
2
Atom notprintedwith(sheet4, front, color)
NegatedAtom notprintedwith(sheet4, front, color)
end_variable
begin_variable
var44
-1
2
Atom sideup(sheet4, front)
NegatedAtom sideup(sheet4, front)
end_variable
begin_variable
var45
-1
2
Atom stackedin(sheet4, finisher1_tray)
NegatedAtom stackedin(sheet4, finisher1_tray)
end_variable
begin_variable
var46
-1
2
Atom sideup(sheet5, back)
NegatedAtom sideup(sheet5, back)
end_variable
begin_variable
var47
-1
2
Atom notprintedwith(sheet5, front, black)
NegatedAtom notprintedwith(sheet5, front, black)
end_variable
begin_variable
var48
-1
18
Atom location(sheet5, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet5, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet5, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet5, colorcontainer_entry-down_bottomexit)
Atom location(sheet5, colorcontainer_exit-up_bottomentry)
Atom location(sheet5, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet5, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet5, down_bottomentry-colorfeeder_exit)
Atom location(sheet5, down_topexit-htmovercolor_entry)
Atom location(sheet5, endcap_entry-blackcontainer_exit)
Atom location(sheet5, finisher1_entry-up_topexit)
Atom location(sheet5, finisher2_entry-finisher1_exit)
Atom location(sheet5, finisher2_exit)
Atom location(sheet5, htmoverblack_entry-endcap_exit)
Atom location(sheet5, htmoverblack_exit-down_topentry)
Atom location(sheet5, htmovercolor_exit-up_topentry)
Atom location(sheet5, some_feeder_tray)
Atom location(sheet5, some_finisher_tray)
end_variable
begin_variable
var49
-1
2
Atom notprintedwith(sheet5, back, black)
NegatedAtom notprintedwith(sheet5, back, black)
end_variable
begin_variable
var50
-1
2
Atom notprintedwith(sheet5, back, color)
NegatedAtom notprintedwith(sheet5, back, color)
end_variable
begin_variable
var51
-1
2
Atom notprintedwith(sheet5, front, color)
NegatedAtom notprintedwith(sheet5, front, color)
end_variable
begin_variable
var52
-1
2
Atom sideup(sheet5, front)
NegatedAtom sideup(sheet5, front)
end_variable
begin_variable
var53
-1
2
Atom stackedin(sheet5, finisher1_tray)
NegatedAtom stackedin(sheet5, finisher1_tray)
end_variable
begin_variable
var54
-1
2
Atom notprintedwith(sheet6, front, color)
NegatedAtom notprintedwith(sheet6, front, color)
end_variable
begin_variable
var55
-1
2
Atom sideup(sheet6, back)
NegatedAtom sideup(sheet6, back)
end_variable
begin_variable
var56
-1
18
Atom location(sheet6, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet6, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet6, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet6, colorcontainer_entry-down_bottomexit)
Atom location(sheet6, colorcontainer_exit-up_bottomentry)
Atom location(sheet6, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet6, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet6, down_bottomentry-colorfeeder_exit)
Atom location(sheet6, down_topexit-htmovercolor_entry)
Atom location(sheet6, endcap_entry-blackcontainer_exit)
Atom location(sheet6, finisher1_entry-up_topexit)
Atom location(sheet6, finisher2_entry-finisher1_exit)
Atom location(sheet6, finisher2_exit)
Atom location(sheet6, htmoverblack_entry-endcap_exit)
Atom location(sheet6, htmoverblack_exit-down_topentry)
Atom location(sheet6, htmovercolor_exit-up_topentry)
Atom location(sheet6, some_feeder_tray)
Atom location(sheet6, some_finisher_tray)
end_variable
begin_variable
var57
-1
2
Atom notprintedwith(sheet6, back, black)
NegatedAtom notprintedwith(sheet6, back, black)
end_variable
begin_variable
var58
-1
2
Atom notprintedwith(sheet6, back, color)
NegatedAtom notprintedwith(sheet6, back, color)
end_variable
begin_variable
var59
-1
2
Atom notprintedwith(sheet6, front, black)
NegatedAtom notprintedwith(sheet6, front, black)
end_variable
begin_variable
var60
-1
2
Atom sideup(sheet6, front)
NegatedAtom sideup(sheet6, front)
end_variable
begin_variable
var61
-1
2
Atom stackedin(sheet6, finisher1_tray)
NegatedAtom stackedin(sheet6, finisher1_tray)
end_variable
begin_variable
var62
-1
2
Atom notprintedwith(sheet7, front, color)
NegatedAtom notprintedwith(sheet7, front, color)
end_variable
begin_variable
var63
-1
2
Atom sideup(sheet7, back)
NegatedAtom sideup(sheet7, back)
end_variable
begin_variable
var64
-1
18
Atom location(sheet7, blackcontainer_entry-blackfeeder_exit)
Atom location(sheet7, blackcontainer_exittoime-blackprinter_entry)
Atom location(sheet7, blackprinter_exit-blackcontainer_entryfromime)
Atom location(sheet7, colorcontainer_entry-down_bottomexit)
Atom location(sheet7, colorcontainer_exit-up_bottomentry)
Atom location(sheet7, colorcontainer_exittoime-colorprinter_entry)
Atom location(sheet7, colorprinter_exit-colorcontainer_entryfromime)
Atom location(sheet7, down_bottomentry-colorfeeder_exit)
Atom location(sheet7, down_topexit-htmovercolor_entry)
Atom location(sheet7, endcap_entry-blackcontainer_exit)
Atom location(sheet7, finisher1_entry-up_topexit)
Atom location(sheet7, finisher2_entry-finisher1_exit)
Atom location(sheet7, finisher2_exit)
Atom location(sheet7, htmoverblack_entry-endcap_exit)
Atom location(sheet7, htmoverblack_exit-down_topentry)
Atom location(sheet7, htmovercolor_exit-up_topentry)
Atom location(sheet7, some_feeder_tray)
Atom location(sheet7, some_finisher_tray)
end_variable
begin_variable
var65
-1
2
Atom notprintedwith(sheet7, back, black)
NegatedAtom notprintedwith(sheet7, back, black)
end_variable
begin_variable
var66
-1
2
Atom notprintedwith(sheet7, back, color)
NegatedAtom notprintedwith(sheet7, back, color)
end_variable
begin_variable
var67
-1
2
Atom notprintedwith(sheet7, front, black)
NegatedAtom notprintedwith(sheet7, front, black)
end_variable
begin_variable
var68
-1
2
Atom sideup(sheet7, front)
NegatedAtom sideup(sheet7, front)
end_variable
begin_variable
var69
-1
2
Atom stackedin(sheet7, finisher1_tray)
NegatedAtom stackedin(sheet7, finisher1_tray)
end_variable
begin_variable
var70
-1
2
Atom hasimage(sheet7, front, image-7)
NegatedAtom hasimage(sheet7, front, image-7)
end_variable
begin_variable
var71
-1
2
Atom hasimage(sheet6, front, image-6)
NegatedAtom hasimage(sheet6, front, image-6)
end_variable
begin_variable
var72
-1
2
Atom hasimage(sheet5, front, image-5)
NegatedAtom hasimage(sheet5, front, image-5)
end_variable
begin_variable
var73
-1
2
Atom hasimage(sheet4, front, image-4)
NegatedAtom hasimage(sheet4, front, image-4)
end_variable
begin_variable
var74
-1
2
Atom hasimage(sheet3, front, image-3)
NegatedAtom hasimage(sheet3, front, image-3)
end_variable
begin_variable
var75
-1
2
Atom hasimage(sheet2, front, image-2)
NegatedAtom hasimage(sheet2, front, image-2)
end_variable
begin_variable
var76
-1
2
Atom hasimage(sheet1, front, image-1)
NegatedAtom hasimage(sheet1, front, image-1)
end_variable
0
begin_state
0
1
1
1
1
1
1
1
1
1
1
1
1
1
1
0
16
0
0
0
1
1
1
0
16
0
0
0
1
1
1
0
16
0
0
0
1
1
1
0
16
0
0
0
1
1
1
0
16
0
0
0
1
1
0
1
16
0
0
0
1
1
0
1
16
0
0
0
1
1
1
1
1
1
1
1
1
end_state
begin_goal
42
17 0
18 0
19 0
20 0
21 0
25 0
26 0
27 0
28 0
29 0
33 0
34 0
35 0
36 0
37 0
41 0
42 0
43 0
44 0
45 0
49 0
50 0
51 0
52 0
53 0
57 0
58 0
59 0
60 0
61 0
65 0
66 0
67 0
68 0
69 0
70 0
71 0
72 0
73 0
74 0
75 0
76 0
end_goal
365
begin_operator
blackcontainer-fromime-letter sheet1
1
13 0
1
0 16 2 9
1
end_operator
begin_operator
blackcontainer-fromime-letter sheet2
1
13 0
1
0 24 2 9
1
end_operator
begin_operator
blackcontainer-fromime-letter sheet3
1
13 0
1
0 32 2 9
1
end_operator
begin_operator
blackcontainer-fromime-letter sheet4
1
13 0
1
0 40 2 9
1
end_operator
begin_operator
blackcontainer-fromime-letter sheet5
1
13 0
1
0 48 2 9
1
end_operator
begin_operator
blackcontainer-fromime-letter sheet6
1
13 0
1
0 56 2 9
1
end_operator
begin_operator
blackcontainer-fromime-letter sheet7
1
13 0
1
0 64 2 9
1
end_operator
begin_operator
blackcontainer-toime-letter sheet1
1
13 0
1
0 16 0 1
1
end_operator
begin_operator
blackcontainer-toime-letter sheet2
1
13 0
1
0 24 0 1
1
end_operator
begin_operator
blackcontainer-toime-letter sheet3
1
13 0
1
0 32 0 1
1
end_operator
begin_operator
blackcontainer-toime-letter sheet4
1
13 0
1
0 40 0 1
1
end_operator
begin_operator
blackcontainer-toime-letter sheet5
1
13 0
1
0 48 0 1
1
end_operator
begin_operator
blackcontainer-toime-letter sheet6
1
13 0
1
0 56 0 1
1
end_operator
begin_operator
blackcontainer-toime-letter sheet7
1
13 0
1
0 64 0 1
1
end_operator
begin_operator
blackfeeder-feed-letter sheet1
1
12 0
2
0 16 16 0
0 20 -1 0
1
end_operator
begin_operator
blackfeeder-feed-letter sheet2
1
12 0
2
0 24 16 0
0 28 -1 0
1
end_operator
begin_operator
blackfeeder-feed-letter sheet3
1
12 0
2
0 32 16 0
0 36 -1 0
1
end_operator
begin_operator
blackfeeder-feed-letter sheet4
1
12 0
2
0 40 16 0
0 44 -1 0
1
end_operator
begin_operator
blackfeeder-feed-letter sheet5
1
12 0
2
0 48 16 0
0 52 -1 0
1
end_operator
begin_operator
blackfeeder-feed-letter sheet6
1
12 0
2
0 56 16 0
0 60 -1 0
1
end_operator
begin_operator
blackfeeder-feed-letter sheet7
1
12 0
2
0 64 16 0
0 68 -1 0
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 back image-1
2
11 0
14 0
2
0 16 1 2
0 17 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 back image-2
2
11 0
14 0
2
0 16 1 2
0 17 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 back image-3
2
11 0
14 0
2
0 16 1 2
0 17 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 back image-4
2
11 0
14 0
2
0 16 1 2
0 17 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 back image-5
2
11 0
14 0
2
0 16 1 2
0 17 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 front image-1
2
11 0
20 0
3
0 76 -1 0
0 16 1 2
0 15 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 front image-2
2
11 0
20 0
2
0 16 1 2
0 15 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 front image-3
2
11 0
20 0
2
0 16 1 2
0 15 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 front image-4
2
11 0
20 0
2
0 16 1 2
0 15 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet1 front image-5
2
11 0
20 0
2
0 16 1 2
0 15 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 back image-1
2
11 0
22 0
2
0 24 1 2
0 25 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 back image-2
2
11 0
22 0
2
0 24 1 2
0 25 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 back image-3
2
11 0
22 0
2
0 24 1 2
0 25 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 back image-4
2
11 0
22 0
2
0 24 1 2
0 25 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 back image-5
2
11 0
22 0
2
0 24 1 2
0 25 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 front image-1
2
11 0
28 0
2
0 24 1 2
0 23 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 front image-2
2
11 0
28 0
3
0 75 -1 0
0 24 1 2
0 23 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 front image-3
2
11 0
28 0
2
0 24 1 2
0 23 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 front image-4
2
11 0
28 0
2
0 24 1 2
0 23 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet2 front image-5
2
11 0
28 0
2
0 24 1 2
0 23 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 back image-1
2
11 0
30 0
2
0 32 1 2
0 33 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 back image-2
2
11 0
30 0
2
0 32 1 2
0 33 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 back image-3
2
11 0
30 0
2
0 32 1 2
0 33 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 back image-4
2
11 0
30 0
2
0 32 1 2
0 33 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 back image-5
2
11 0
30 0
2
0 32 1 2
0 33 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 front image-1
2
11 0
36 0
2
0 32 1 2
0 31 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 front image-2
2
11 0
36 0
2
0 32 1 2
0 31 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 front image-3
2
11 0
36 0
3
0 74 -1 0
0 32 1 2
0 31 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 front image-4
2
11 0
36 0
2
0 32 1 2
0 31 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet3 front image-5
2
11 0
36 0
2
0 32 1 2
0 31 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 back image-1
2
11 0
38 0
2
0 40 1 2
0 41 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 back image-2
2
11 0
38 0
2
0 40 1 2
0 41 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 back image-3
2
11 0
38 0
2
0 40 1 2
0 41 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 back image-4
2
11 0
38 0
2
0 40 1 2
0 41 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 back image-5
2
11 0
38 0
2
0 40 1 2
0 41 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 front image-1
2
11 0
44 0
2
0 40 1 2
0 39 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 front image-2
2
11 0
44 0
2
0 40 1 2
0 39 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 front image-3
2
11 0
44 0
2
0 40 1 2
0 39 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 front image-4
2
11 0
44 0
3
0 73 -1 0
0 40 1 2
0 39 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet4 front image-5
2
11 0
44 0
2
0 40 1 2
0 39 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 back image-1
2
11 0
46 0
2
0 48 1 2
0 49 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 back image-2
2
11 0
46 0
2
0 48 1 2
0 49 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 back image-3
2
11 0
46 0
2
0 48 1 2
0 49 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 back image-4
2
11 0
46 0
2
0 48 1 2
0 49 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 back image-5
2
11 0
46 0
2
0 48 1 2
0 49 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 front image-1
2
11 0
52 0
2
0 48 1 2
0 47 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 front image-2
2
11 0
52 0
2
0 48 1 2
0 47 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 front image-3
2
11 0
52 0
2
0 48 1 2
0 47 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 front image-4
2
11 0
52 0
2
0 48 1 2
0 47 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet5 front image-5
2
11 0
52 0
3
0 72 -1 0
0 48 1 2
0 47 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 back image-1
2
11 0
55 0
2
0 56 1 2
0 57 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 back image-2
2
11 0
55 0
2
0 56 1 2
0 57 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 back image-3
2
11 0
55 0
2
0 56 1 2
0 57 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 back image-4
2
11 0
55 0
2
0 56 1 2
0 57 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 back image-5
2
11 0
55 0
2
0 56 1 2
0 57 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 front image-1
2
11 0
60 0
2
0 56 1 2
0 59 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 front image-2
2
11 0
60 0
2
0 56 1 2
0 59 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 front image-3
2
11 0
60 0
2
0 56 1 2
0 59 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 front image-4
2
11 0
60 0
2
0 56 1 2
0 59 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet6 front image-5
2
11 0
60 0
2
0 56 1 2
0 59 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 back image-1
2
11 0
63 0
2
0 64 1 2
0 65 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 back image-2
2
11 0
63 0
2
0 64 1 2
0 65 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 back image-3
2
11 0
63 0
2
0 64 1 2
0 65 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 back image-4
2
11 0
63 0
2
0 64 1 2
0 65 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 back image-5
2
11 0
63 0
2
0 64 1 2
0 65 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 front image-1
2
11 0
68 0
2
0 64 1 2
0 67 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 front image-2
2
11 0
68 0
2
0 64 1 2
0 67 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 front image-3
2
11 0
68 0
2
0 64 1 2
0 67 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 front image-4
2
11 0
68 0
2
0 64 1 2
0 67 0 1
1
end_operator
begin_operator
blackprinter-simplex-letter sheet7 front image-5
2
11 0
68 0
2
0 64 1 2
0 67 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 back front image-1
1
11 0
4
0 16 1 2
0 17 0 1
0 14 0 1
0 20 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 back front image-2
1
11 0
4
0 16 1 2
0 17 0 1
0 14 0 1
0 20 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 back front image-3
1
11 0
4
0 16 1 2
0 17 0 1
0 14 0 1
0 20 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 back front image-4
1
11 0
4
0 16 1 2
0 17 0 1
0 14 0 1
0 20 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 back front image-5
1
11 0
4
0 16 1 2
0 17 0 1
0 14 0 1
0 20 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 front back image-1
1
11 0
5
0 76 -1 0
0 16 1 2
0 15 0 1
0 14 -1 0
0 20 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 front back image-2
1
11 0
4
0 16 1 2
0 15 0 1
0 14 -1 0
0 20 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 front back image-3
1
11 0
4
0 16 1 2
0 15 0 1
0 14 -1 0
0 20 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 front back image-4
1
11 0
4
0 16 1 2
0 15 0 1
0 14 -1 0
0 20 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet1 front back image-5
1
11 0
4
0 16 1 2
0 15 0 1
0 14 -1 0
0 20 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 back front image-1
1
11 0
4
0 24 1 2
0 25 0 1
0 22 0 1
0 28 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 back front image-2
1
11 0
4
0 24 1 2
0 25 0 1
0 22 0 1
0 28 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 back front image-3
1
11 0
4
0 24 1 2
0 25 0 1
0 22 0 1
0 28 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 back front image-4
1
11 0
4
0 24 1 2
0 25 0 1
0 22 0 1
0 28 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 back front image-5
1
11 0
4
0 24 1 2
0 25 0 1
0 22 0 1
0 28 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 front back image-1
1
11 0
4
0 24 1 2
0 23 0 1
0 22 -1 0
0 28 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 front back image-2
1
11 0
5
0 75 -1 0
0 24 1 2
0 23 0 1
0 22 -1 0
0 28 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 front back image-3
1
11 0
4
0 24 1 2
0 23 0 1
0 22 -1 0
0 28 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 front back image-4
1
11 0
4
0 24 1 2
0 23 0 1
0 22 -1 0
0 28 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet2 front back image-5
1
11 0
4
0 24 1 2
0 23 0 1
0 22 -1 0
0 28 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 back front image-1
1
11 0
4
0 32 1 2
0 33 0 1
0 30 0 1
0 36 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 back front image-2
1
11 0
4
0 32 1 2
0 33 0 1
0 30 0 1
0 36 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 back front image-3
1
11 0
4
0 32 1 2
0 33 0 1
0 30 0 1
0 36 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 back front image-4
1
11 0
4
0 32 1 2
0 33 0 1
0 30 0 1
0 36 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 back front image-5
1
11 0
4
0 32 1 2
0 33 0 1
0 30 0 1
0 36 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 front back image-1
1
11 0
4
0 32 1 2
0 31 0 1
0 30 -1 0
0 36 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 front back image-2
1
11 0
4
0 32 1 2
0 31 0 1
0 30 -1 0
0 36 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 front back image-3
1
11 0
5
0 74 -1 0
0 32 1 2
0 31 0 1
0 30 -1 0
0 36 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 front back image-4
1
11 0
4
0 32 1 2
0 31 0 1
0 30 -1 0
0 36 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet3 front back image-5
1
11 0
4
0 32 1 2
0 31 0 1
0 30 -1 0
0 36 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 back front image-1
1
11 0
4
0 40 1 2
0 41 0 1
0 38 0 1
0 44 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 back front image-2
1
11 0
4
0 40 1 2
0 41 0 1
0 38 0 1
0 44 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 back front image-3
1
11 0
4
0 40 1 2
0 41 0 1
0 38 0 1
0 44 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 back front image-4
1
11 0
4
0 40 1 2
0 41 0 1
0 38 0 1
0 44 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 back front image-5
1
11 0
4
0 40 1 2
0 41 0 1
0 38 0 1
0 44 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 front back image-1
1
11 0
4
0 40 1 2
0 39 0 1
0 38 -1 0
0 44 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 front back image-2
1
11 0
4
0 40 1 2
0 39 0 1
0 38 -1 0
0 44 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 front back image-3
1
11 0
4
0 40 1 2
0 39 0 1
0 38 -1 0
0 44 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 front back image-4
1
11 0
5
0 73 -1 0
0 40 1 2
0 39 0 1
0 38 -1 0
0 44 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet4 front back image-5
1
11 0
4
0 40 1 2
0 39 0 1
0 38 -1 0
0 44 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 back front image-1
1
11 0
4
0 48 1 2
0 49 0 1
0 46 0 1
0 52 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 back front image-2
1
11 0
4
0 48 1 2
0 49 0 1
0 46 0 1
0 52 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 back front image-3
1
11 0
4
0 48 1 2
0 49 0 1
0 46 0 1
0 52 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 back front image-4
1
11 0
4
0 48 1 2
0 49 0 1
0 46 0 1
0 52 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 back front image-5
1
11 0
4
0 48 1 2
0 49 0 1
0 46 0 1
0 52 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 front back image-1
1
11 0
4
0 48 1 2
0 47 0 1
0 46 -1 0
0 52 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 front back image-2
1
11 0
4
0 48 1 2
0 47 0 1
0 46 -1 0
0 52 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 front back image-3
1
11 0
4
0 48 1 2
0 47 0 1
0 46 -1 0
0 52 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 front back image-4
1
11 0
4
0 48 1 2
0 47 0 1
0 46 -1 0
0 52 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet5 front back image-5
1
11 0
5
0 72 -1 0
0 48 1 2
0 47 0 1
0 46 -1 0
0 52 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 back front image-1
1
11 0
4
0 56 1 2
0 57 0 1
0 55 0 1
0 60 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 back front image-2
1
11 0
4
0 56 1 2
0 57 0 1
0 55 0 1
0 60 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 back front image-3
1
11 0
4
0 56 1 2
0 57 0 1
0 55 0 1
0 60 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 back front image-4
1
11 0
4
0 56 1 2
0 57 0 1
0 55 0 1
0 60 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 back front image-5
1
11 0
4
0 56 1 2
0 57 0 1
0 55 0 1
0 60 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 front back image-1
1
11 0
4
0 56 1 2
0 59 0 1
0 55 -1 0
0 60 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 front back image-2
1
11 0
4
0 56 1 2
0 59 0 1
0 55 -1 0
0 60 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 front back image-3
1
11 0
4
0 56 1 2
0 59 0 1
0 55 -1 0
0 60 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 front back image-4
1
11 0
4
0 56 1 2
0 59 0 1
0 55 -1 0
0 60 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet6 front back image-5
1
11 0
4
0 56 1 2
0 59 0 1
0 55 -1 0
0 60 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 back front image-1
1
11 0
4
0 64 1 2
0 65 0 1
0 63 0 1
0 68 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 back front image-2
1
11 0
4
0 64 1 2
0 65 0 1
0 63 0 1
0 68 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 back front image-3
1
11 0
4
0 64 1 2
0 65 0 1
0 63 0 1
0 68 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 back front image-4
1
11 0
4
0 64 1 2
0 65 0 1
0 63 0 1
0 68 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 back front image-5
1
11 0
4
0 64 1 2
0 65 0 1
0 63 0 1
0 68 -1 0
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 front back image-1
1
11 0
4
0 64 1 2
0 67 0 1
0 63 -1 0
0 68 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 front back image-2
1
11 0
4
0 64 1 2
0 67 0 1
0 63 -1 0
0 68 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 front back image-3
1
11 0
4
0 64 1 2
0 67 0 1
0 63 -1 0
0 68 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 front back image-4
1
11 0
4
0 64 1 2
0 67 0 1
0 63 -1 0
0 68 0 1
1
end_operator
begin_operator
blackprinter-simplexandinvert-letter sheet7 front back image-5
1
11 0
4
0 64 1 2
0 67 0 1
0 63 -1 0
0 68 0 1
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet1
1
10 0
1
0 16 6 4
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet2
1
10 0
1
0 24 6 4
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet3
1
10 0
1
0 32 6 4
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet4
1
10 0
1
0 40 6 4
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet5
1
10 0
1
0 48 6 4
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet6
1
10 0
1
0 56 6 4
1
end_operator
begin_operator
colorcontainer-fromime-letter sheet7
1
10 0
1
0 64 6 4
1
end_operator
begin_operator
colorcontainer-toime-letter sheet1
1
10 0
1
0 16 3 5
1
end_operator
begin_operator
colorcontainer-toime-letter sheet2
1
10 0
1
0 24 3 5
1
end_operator
begin_operator
colorcontainer-toime-letter sheet3
1
10 0
1
0 32 3 5
1
end_operator
begin_operator
colorcontainer-toime-letter sheet4
1
10 0
1
0 40 3 5
1
end_operator
begin_operator
colorcontainer-toime-letter sheet5
1
10 0
1
0 48 3 5
1
end_operator
begin_operator
colorcontainer-toime-letter sheet6
1
10 0
1
0 56 3 5
1
end_operator
begin_operator
colorcontainer-toime-letter sheet7
1
10 0
1
0 64 3 5
1
end_operator
begin_operator
colorfeeder-feed-letter sheet1
1
9 0
2
0 16 16 7
0 20 -1 0
1
end_operator
begin_operator
colorfeeder-feed-letter sheet2
1
9 0
2
0 24 16 7
0 28 -1 0
1
end_operator
begin_operator
colorfeeder-feed-letter sheet3
1
9 0
2
0 32 16 7
0 36 -1 0
1
end_operator
begin_operator
colorfeeder-feed-letter sheet4
1
9 0
2
0 40 16 7
0 44 -1 0
1
end_operator
begin_operator
colorfeeder-feed-letter sheet5
1
9 0
2
0 48 16 7
0 52 -1 0
1
end_operator
begin_operator
colorfeeder-feed-letter sheet6
1
9 0
2
0 56 16 7
0 60 -1 0
1
end_operator
begin_operator
colorfeeder-feed-letter sheet7
1
9 0
2
0 64 16 7
0 68 -1 0
1
end_operator
begin_operator
colorprinter-simplex-letter sheet1 back image-6
2
8 0
14 0
2
0 16 5 6
0 18 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet1 back image-7
2
8 0
14 0
2
0 16 5 6
0 18 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet1 front image-6
2
8 0
20 0
2
0 16 5 6
0 19 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet1 front image-7
2
8 0
20 0
2
0 16 5 6
0 19 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet2 back image-6
2
8 0
22 0
2
0 24 5 6
0 26 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet2 back image-7
2
8 0
22 0
2
0 24 5 6
0 26 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet2 front image-6
2
8 0
28 0
2
0 24 5 6
0 27 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet2 front image-7
2
8 0
28 0
2
0 24 5 6
0 27 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet3 back image-6
2
8 0
30 0
2
0 32 5 6
0 34 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet3 back image-7
2
8 0
30 0
2
0 32 5 6
0 34 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet3 front image-6
2
8 0
36 0
2
0 32 5 6
0 35 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet3 front image-7
2
8 0
36 0
2
0 32 5 6
0 35 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet4 back image-6
2
8 0
38 0
2
0 40 5 6
0 42 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet4 back image-7
2
8 0
38 0
2
0 40 5 6
0 42 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet4 front image-6
2
8 0
44 0
2
0 40 5 6
0 43 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet4 front image-7
2
8 0
44 0
2
0 40 5 6
0 43 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet5 back image-6
2
8 0
46 0
2
0 48 5 6
0 50 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet5 back image-7
2
8 0
46 0
2
0 48 5 6
0 50 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet5 front image-6
2
8 0
52 0
2
0 48 5 6
0 51 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet5 front image-7
2
8 0
52 0
2
0 48 5 6
0 51 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet6 back image-6
2
8 0
55 0
2
0 56 5 6
0 58 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet6 back image-7
2
8 0
55 0
2
0 56 5 6
0 58 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet6 front image-6
2
8 0
60 0
3
0 71 -1 0
0 56 5 6
0 54 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet6 front image-7
2
8 0
60 0
2
0 56 5 6
0 54 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet7 back image-6
2
8 0
63 0
2
0 64 5 6
0 66 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet7 back image-7
2
8 0
63 0
2
0 64 5 6
0 66 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet7 front image-6
2
8 0
68 0
2
0 64 5 6
0 62 0 1
1
end_operator
begin_operator
colorprinter-simplex-letter sheet7 front image-7
2
8 0
68 0
3
0 70 -1 0
0 64 5 6
0 62 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 back image-1
2
8 0
14 0
2
0 16 5 6
0 17 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 back image-2
2
8 0
14 0
2
0 16 5 6
0 17 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 back image-3
2
8 0
14 0
2
0 16 5 6
0 17 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 back image-4
2
8 0
14 0
2
0 16 5 6
0 17 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 back image-5
2
8 0
14 0
2
0 16 5 6
0 17 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 front image-1
2
8 0
20 0
3
0 76 -1 0
0 16 5 6
0 15 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 front image-2
2
8 0
20 0
2
0 16 5 6
0 15 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 front image-3
2
8 0
20 0
2
0 16 5 6
0 15 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 front image-4
2
8 0
20 0
2
0 16 5 6
0 15 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet1 front image-5
2
8 0
20 0
2
0 16 5 6
0 15 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 back image-1
2
8 0
22 0
2
0 24 5 6
0 25 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 back image-2
2
8 0
22 0
2
0 24 5 6
0 25 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 back image-3
2
8 0
22 0
2
0 24 5 6
0 25 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 back image-4
2
8 0
22 0
2
0 24 5 6
0 25 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 back image-5
2
8 0
22 0
2
0 24 5 6
0 25 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 front image-1
2
8 0
28 0
2
0 24 5 6
0 23 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 front image-2
2
8 0
28 0
3
0 75 -1 0
0 24 5 6
0 23 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 front image-3
2
8 0
28 0
2
0 24 5 6
0 23 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 front image-4
2
8 0
28 0
2
0 24 5 6
0 23 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet2 front image-5
2
8 0
28 0
2
0 24 5 6
0 23 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 back image-1
2
8 0
30 0
2
0 32 5 6
0 33 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 back image-2
2
8 0
30 0
2
0 32 5 6
0 33 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 back image-3
2
8 0
30 0
2
0 32 5 6
0 33 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 back image-4
2
8 0
30 0
2
0 32 5 6
0 33 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 back image-5
2
8 0
30 0
2
0 32 5 6
0 33 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 front image-1
2
8 0
36 0
2
0 32 5 6
0 31 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 front image-2
2
8 0
36 0
2
0 32 5 6
0 31 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 front image-3
2
8 0
36 0
3
0 74 -1 0
0 32 5 6
0 31 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 front image-4
2
8 0
36 0
2
0 32 5 6
0 31 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet3 front image-5
2
8 0
36 0
2
0 32 5 6
0 31 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 back image-1
2
8 0
38 0
2
0 40 5 6
0 41 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 back image-2
2
8 0
38 0
2
0 40 5 6
0 41 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 back image-3
2
8 0
38 0
2
0 40 5 6
0 41 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 back image-4
2
8 0
38 0
2
0 40 5 6
0 41 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 back image-5
2
8 0
38 0
2
0 40 5 6
0 41 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 front image-1
2
8 0
44 0
2
0 40 5 6
0 39 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 front image-2
2
8 0
44 0
2
0 40 5 6
0 39 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 front image-3
2
8 0
44 0
2
0 40 5 6
0 39 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 front image-4
2
8 0
44 0
3
0 73 -1 0
0 40 5 6
0 39 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet4 front image-5
2
8 0
44 0
2
0 40 5 6
0 39 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 back image-1
2
8 0
46 0
2
0 48 5 6
0 49 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 back image-2
2
8 0
46 0
2
0 48 5 6
0 49 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 back image-3
2
8 0
46 0
2
0 48 5 6
0 49 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 back image-4
2
8 0
46 0
2
0 48 5 6
0 49 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 back image-5
2
8 0
46 0
2
0 48 5 6
0 49 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 front image-1
2
8 0
52 0
2
0 48 5 6
0 47 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 front image-2
2
8 0
52 0
2
0 48 5 6
0 47 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 front image-3
2
8 0
52 0
2
0 48 5 6
0 47 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 front image-4
2
8 0
52 0
2
0 48 5 6
0 47 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet5 front image-5
2
8 0
52 0
3
0 72 -1 0
0 48 5 6
0 47 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 back image-1
2
8 0
55 0
2
0 56 5 6
0 57 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 back image-2
2
8 0
55 0
2
0 56 5 6
0 57 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 back image-3
2
8 0
55 0
2
0 56 5 6
0 57 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 back image-4
2
8 0
55 0
2
0 56 5 6
0 57 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 back image-5
2
8 0
55 0
2
0 56 5 6
0 57 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 front image-1
2
8 0
60 0
2
0 56 5 6
0 59 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 front image-2
2
8 0
60 0
2
0 56 5 6
0 59 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 front image-3
2
8 0
60 0
2
0 56 5 6
0 59 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 front image-4
2
8 0
60 0
2
0 56 5 6
0 59 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet6 front image-5
2
8 0
60 0
2
0 56 5 6
0 59 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 back image-1
2
8 0
63 0
2
0 64 5 6
0 65 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 back image-2
2
8 0
63 0
2
0 64 5 6
0 65 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 back image-3
2
8 0
63 0
2
0 64 5 6
0 65 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 back image-4
2
8 0
63 0
2
0 64 5 6
0 65 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 back image-5
2
8 0
63 0
2
0 64 5 6
0 65 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 front image-1
2
8 0
68 0
2
0 64 5 6
0 67 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 front image-2
2
8 0
68 0
2
0 64 5 6
0 67 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 front image-3
2
8 0
68 0
2
0 64 5 6
0 67 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 front image-4
2
8 0
68 0
2
0 64 5 6
0 67 0 1
1
end_operator
begin_operator
colorprinter-simplexmono-letter sheet7 front image-5
2
8 0
68 0
2
0 64 5 6
0 67 0 1
1
end_operator
begin_operator
down-movebottom-letter sheet1
1
7 0
1
0 16 7 3
1
end_operator
begin_operator
down-movebottom-letter sheet2
1
7 0
1
0 24 7 3
1
end_operator
begin_operator
down-movebottom-letter sheet3
1
7 0
1
0 32 7 3
1
end_operator
begin_operator
down-movebottom-letter sheet4
1
7 0
1
0 40 7 3
1
end_operator
begin_operator
down-movebottom-letter sheet5
1
7 0
1
0 48 7 3
1
end_operator
begin_operator
down-movebottom-letter sheet6
1
7 0
1
0 56 7 3
1
end_operator
begin_operator
down-movebottom-letter sheet7
1
7 0
1
0 64 7 3
1
end_operator
begin_operator
down-movedown-letter sheet1
1
7 0
1
0 16 14 3
1
end_operator
begin_operator
down-movedown-letter sheet2
1
7 0
1
0 24 14 3
1
end_operator
begin_operator
down-movedown-letter sheet3
1
7 0
1
0 32 14 3
1
end_operator
begin_operator
down-movedown-letter sheet4
1
7 0
1
0 40 14 3
1
end_operator
begin_operator
down-movedown-letter sheet5
1
7 0
1
0 48 14 3
1
end_operator
begin_operator
down-movedown-letter sheet6
1
7 0
1
0 56 14 3
1
end_operator
begin_operator
down-movedown-letter sheet7
1
7 0
1
0 64 14 3
1
end_operator
begin_operator
down-movetop-letter sheet1
1
7 0
1
0 16 14 8
1
end_operator
begin_operator
down-movetop-letter sheet2
1
7 0
1
0 24 14 8
1
end_operator
begin_operator
down-movetop-letter sheet3
1
7 0
1
0 32 14 8
1
end_operator
begin_operator
down-movetop-letter sheet4
1
7 0
1
0 40 14 8
1
end_operator
begin_operator
down-movetop-letter sheet5
1
7 0
1
0 48 14 8
1
end_operator
begin_operator
down-movetop-letter sheet6
1
7 0
1
0 56 14 8
1
end_operator
begin_operator
down-movetop-letter sheet7
1
7 0
1
0 64 14 8
1
end_operator
begin_operator
endcap-move-letter sheet1
1
6 0
1
0 16 9 13
1
end_operator
begin_operator
endcap-move-letter sheet2
1
6 0
1
0 24 9 13
1
end_operator
begin_operator
endcap-move-letter sheet3
1
6 0
1
0 32 9 13
1
end_operator
begin_operator
endcap-move-letter sheet4
1
6 0
1
0 40 9 13
1
end_operator
begin_operator
endcap-move-letter sheet5
1
6 0
1
0 48 9 13
1
end_operator
begin_operator
endcap-move-letter sheet6
1
6 0
1
0 56 9 13
1
end_operator
begin_operator
endcap-move-letter sheet7
1
6 0
1
0 64 9 13
1
end_operator
begin_operator
finisher1-passthrough-letter sheet1
1
5 0
1
0 16 10 11
1
end_operator
begin_operator
finisher1-passthrough-letter sheet2
1
5 0
1
0 24 10 11
1
end_operator
begin_operator
finisher1-passthrough-letter sheet3
1
5 0
1
0 32 10 11
1
end_operator
begin_operator
finisher1-passthrough-letter sheet4
1
5 0
1
0 40 10 11
1
end_operator
begin_operator
finisher1-passthrough-letter sheet5
1
5 0
1
0 48 10 11
1
end_operator
begin_operator
finisher1-passthrough-letter sheet6
1
5 0
1
0 56 10 11
1
end_operator
begin_operator
finisher1-passthrough-letter sheet7
1
5 0
1
0 64 10 11
1
end_operator
begin_operator
finisher1-stack-letter sheet1 dummy-sheet
1
5 0
2
0 16 10 17
0 21 -1 0
1
end_operator
begin_operator
finisher1-stack-letter sheet2 sheet1
2
5 0
16 17
2
0 24 10 17
0 29 -1 0
1
end_operator
begin_operator
finisher1-stack-letter sheet3 sheet2
2
5 0
24 17
2
0 32 10 17
0 37 -1 0
1
end_operator
begin_operator
finisher1-stack-letter sheet4 sheet3
2
5 0
32 17
2
0 40 10 17
0 45 -1 0
1
end_operator
begin_operator
finisher1-stack-letter sheet5 sheet4
2
5 0
40 17
2
0 48 10 17
0 53 -1 0
1
end_operator
begin_operator
finisher1-stack-letter sheet6 sheet5
2
5 0
48 17
2
0 56 10 17
0 61 -1 0
1
end_operator
begin_operator
finisher1-stack-letter sheet7 sheet6
2
5 0
56 17
2
0 64 10 17
0 69 -1 0
1
end_operator
begin_operator
finisher2-passthrough-letter sheet1
1
4 0
1
0 16 11 12
1
end_operator
begin_operator
finisher2-passthrough-letter sheet2
1
4 0
1
0 24 11 12
1
end_operator
begin_operator
finisher2-passthrough-letter sheet3
1
4 0
1
0 32 11 12
1
end_operator
begin_operator
finisher2-passthrough-letter sheet4
1
4 0
1
0 40 11 12
1
end_operator
begin_operator
finisher2-passthrough-letter sheet5
1
4 0
1
0 48 11 12
1
end_operator
begin_operator
finisher2-passthrough-letter sheet6
1
4 0
1
0 56 11 12
1
end_operator
begin_operator
finisher2-passthrough-letter sheet7
1
4 0
1
0 64 11 12
1
end_operator
begin_operator
finisher2-stack-letter sheet1 dummy-sheet
1
4 0
1
0 16 11 17
1
end_operator
begin_operator
finisher2-stack-letter sheet2 sheet1
2
4 0
16 17
1
0 24 11 17
1
end_operator
begin_operator
finisher2-stack-letter sheet3 sheet2
2
4 0
24 17
1
0 32 11 17
1
end_operator
begin_operator
finisher2-stack-letter sheet4 sheet3
2
4 0
32 17
1
0 40 11 17
1
end_operator
begin_operator
finisher2-stack-letter sheet5 sheet4
2
4 0
40 17
1
0 48 11 17
1
end_operator
begin_operator
finisher2-stack-letter sheet6 sheet5
2
4 0
48 17
1
0 56 11 17
1
end_operator
begin_operator
finisher2-stack-letter sheet7 sheet6
2
4 0
56 17
1
0 64 11 17
1
end_operator
begin_operator
htmoverblack-move-letter sheet1
1
3 0
1
0 16 13 14
1
end_operator
begin_operator
htmoverblack-move-letter sheet2
1
3 0
1
0 24 13 14
1
end_operator
begin_operator
htmoverblack-move-letter sheet3
1
3 0
1
0 32 13 14
1
end_operator
begin_operator
htmoverblack-move-letter sheet4
1
3 0
1
0 40 13 14
1
end_operator
begin_operator
htmoverblack-move-letter sheet5
1
3 0
1
0 48 13 14
1
end_operator
begin_operator
htmoverblack-move-letter sheet6
1
3 0
1
0 56 13 14
1
end_operator
begin_operator
htmoverblack-move-letter sheet7
1
3 0
1
0 64 13 14
1
end_operator
begin_operator
htmovercolor-move-letter sheet1
1
2 0
1
0 16 8 15
1
end_operator
begin_operator
htmovercolor-move-letter sheet2
1
2 0
1
0 24 8 15
1
end_operator
begin_operator
htmovercolor-move-letter sheet3
1
2 0
1
0 32 8 15
1
end_operator
begin_operator
htmovercolor-move-letter sheet4
1
2 0
1
0 40 8 15
1
end_operator
begin_operator
htmovercolor-move-letter sheet5
1
2 0
1
0 48 8 15
1
end_operator
begin_operator
htmovercolor-move-letter sheet6
1
2 0
1
0 56 8 15
1
end_operator
begin_operator
htmovercolor-move-letter sheet7
1
2 0
1
0 64 8 15
1
end_operator
begin_operator
initialize 
0
14
0 13 -1 0
0 12 -1 0
0 11 -1 0
0 10 -1 0
0 9 -1 0
0 8 -1 0
0 7 -1 0
0 6 -1 0
0 5 -1 0
0 4 -1 0
0 3 -1 0
0 2 -1 0
0 1 -1 0
0 0 0 1
1
end_operator
begin_operator
up-movetop-letter sheet1
1
1 0
1
0 16 15 10
1
end_operator
begin_operator
up-movetop-letter sheet2
1
1 0
1
0 24 15 10
1
end_operator
begin_operator
up-movetop-letter sheet3
1
1 0
1
0 32 15 10
1
end_operator
begin_operator
up-movetop-letter sheet4
1
1 0
1
0 40 15 10
1
end_operator
begin_operator
up-movetop-letter sheet5
1
1 0
1
0 48 15 10
1
end_operator
begin_operator
up-movetop-letter sheet6
1
1 0
1
0 56 15 10
1
end_operator
begin_operator
up-movetop-letter sheet7
1
1 0
1
0 64 15 10
1
end_operator
begin_operator
up-moveup-letter sheet1
1
1 0
1
0 16 4 10
1
end_operator
begin_operator
up-moveup-letter sheet2
1
1 0
1
0 24 4 10
1
end_operator
begin_operator
up-moveup-letter sheet3
1
1 0
1
0 32 4 10
1
end_operator
begin_operator
up-moveup-letter sheet4
1
1 0
1
0 40 4 10
1
end_operator
begin_operator
up-moveup-letter sheet5
1
1 0
1
0 48 4 10
1
end_operator
begin_operator
up-moveup-letter sheet6
1
1 0
1
0 56 4 10
1
end_operator
begin_operator
up-moveup-letter sheet7
1
1 0
1
0 64 4 10
1
end_operator
0
